package com.xhkjedu.slive.service.xz;

import com.xhkjedu.slive.mapper.gradeclass.ClassTeacherMapper;
import com.xhkjedu.slive.mapper.gradeclass.SchoolMapper;
import com.xhkjedu.slive.mapper.liveplay.CourseClassMapper;
import com.xhkjedu.slive.mapper.liveplay.CourseMapper;
import com.xhkjedu.slive.mapper.liveplay.CourseSectionMapper;
import com.xhkjedu.slive.mapper.xz.XzKjMapper;
import com.xhkjedu.slive.mapper.xz.XzMapper;
import com.xhkjedu.slive.model.gradeclass.TClass;
import com.xhkjedu.slive.model.gradeclass.TClassTeacher;
import com.xhkjedu.slive.model.liveplay.LCourse;
import com.xhkjedu.slive.model.liveplay.LCourseClass;
import com.xhkjedu.slive.model.liveplay.LCourseSection;
import com.xhkjedu.slive.model.xz.LXz;
import com.xhkjedu.slive.model.xz.LXzKj;
import com.xhkjedu.slive.service.gradeclass.ClassService;
import com.xhkjedu.slive.utils.LiveUtil;
import com.xhkjedu.slive.vo.gradeclass.ClassVo;
import com.xhkjedu.slive.vo.xz.XzDateVo;
import com.xhkjedu.slive.vo.xz.XzDateXjTeaVo;
import com.xhkjedu.slive.vo.xz.XzDateXjVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 行政直播概要
 * @Author WN
 * Date 2022/3/30 13:42
 **/
@Service
@Slf4j
public class XzService {
    @Resource
    private XzMapper xzMapper;
    @Resource
    private XzKjMapper xzKjMapper;
    @Resource
    private CourseMapper courseMapper;
    @Resource
    private CourseSectionMapper courseSectionMapper;
    @Resource
    private CourseClassMapper courseClassMapper;
    @Resource
    private SchoolMapper schoolMapper;
    @Resource
    private ClassService classService;
    @Resource
    private ClassTeacherMapper classTeacherMapper;

    /**
     * @Description 行政直播列表
     * @Param [schoolid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/3/30 14:03
     **/
    public List<Map> listXzs(LXz xz) {
        return xzMapper.listXzs(xz);
    }

    /**
     * @Description 保存行政直播
     * @Param [xz]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 14:07
     **/
    @Transactional
    public Integer saveXz(LXz xz) {
        Integer timestamp = N_Utils.getSecondTimestamp();
        xz.setCreatetime(timestamp);
        xz.setDeleted(1);
        xzMapper.insertUseGeneratedKeys(xz);//保存行政直播信息
        Integer xzid = xz.getId();
        //保存对应的课节信息
        List<LXzKj> kjlst = new ArrayList<>();
        for (int i = 0; i < xz.getDaynum(); i++) {
            LXzKj kj = new LXzKj();
            kj.setXzid(xzid);
            String kjname = LiveUtil.getChineStrByNum(i);
            kj.setKjname("第" + kjname + "节");
            kj.setKjorder(i+1);
            kj.setCreateid(xz.getCreateid());
            kj.setCreatetime(timestamp);

            kjlst.add(kj);
        }
        //批量保存行政直播对应的课节
        xzKjMapper.insertList(kjlst);
        return xzid;
    }

    /**
     * @Description 修改行政直播
     * @Param [xz]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 14:16
     **/
    public void updateXz(LXz xz) {
        xzMapper.updateByPrimaryKeySelective(xz);
    }

    //获取行政直播课下进行中或者已结束的直播课数量
    public Integer getXzSectionKsNum(Integer xzid) {
        Integer num = xzMapper.getXzSectionKsNum(xzid);
        if (num == null) {
            num = 0;
        }
        return num;
    }

    /**
     * @Description 删除行政直播
     * @Param [xzid]
     * @Return void
     * @Author wn
     * @Date 2022/3/30 15:24
     **/
    @Transactional(rollbackFor = Exception.class)
    public void deleteXz(Integer xzid) {
        try {
            //如果行政直播下所有的课都未开始，则可以直接删除，
            //如果有直播课已经开始则进行逻辑删除，并删除所有未开始上的直播课
            Integer sectionnum = xzMapper.getXzSectionKsNum(xzid);
            if (sectionnum != null && sectionnum > 0) {
                //说明直播下有开始或结束的课节，只能删除未开始的直播和直播班级信息
                xzMapper.deleteCourseClassKjByXzid(xzid);
                //修改行政直播的状态未已删除
                xzMapper.updateXzDeleted(10, xzid);
            } else {
                //说明该行政直播课程可以直接删除
                xzMapper.deleteCourseClassByXzid(xzid);
                xzMapper.deleteXzAndKjByXzid(xzid);
            }
        }catch (Exception e){
            log.error("删除行政直播出错："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }


    }
    /**
     * @Description 保存行政直播课程表
     * @Param [xzDatelst]
     * @Return com.xhkjedu.vo.ResultVo
     * @Author wn
     * @Date 2022/3/31 21:46
     **/
    public ResultVo saveXzCourse(LXz xz) {
        //保存行政直播课程表,参数层级为日期--课节--班级和教师
        //根据同一个课节下根据班级和教师确认是否是同一个直播课，保存进course,section（单课节课程）
        LXz x = xzMapper.selectByPrimaryKey(xz.getXzid());
        List<XzDateVo> xzDatelst = xz.getXzdatas();
        for (int i = 0; i < xzDatelst.size(); i++) {
            //日期下的课节情况
            XzDateVo xzDateVo = xzDatelst.get(i);
            String datazhou = LiveUtil.getDateZhou(xzDateVo.getCurrentdate());//要保存日期对应的周
            List<XzDateXjVo> dateKjlst = xzDateVo.getKjs();//指定日期下课节情况

            for (int j = 0; j < dateKjlst.size(); j++) {
                //单课节情况
                XzDateXjVo dateKjVo = dateKjlst.get(j);
                List<XzDateXjTeaVo> xjTealst = dateKjVo.getKjteas();//单课节下各班级情况
                //处理成教师下班级形式，生成课程
                Map<Integer, List<XzDateXjTeaVo>> map = listChangeTeacherClass(xjTealst);
                LCourse course = new LCourse();
                course.setGradeid(x.getGradeid());
                course.setBegindate(x.getBegindate());
                course.setEnddate(x.getEnddate());
                course.setCoursenum(1);
                course.setCoursetype(1);
                course.setCourseclass(1);
                course.setCoursestate(0);
                course.setDeleted(1);
                course.setCreateid(xz.getCreateid());
                course.setCreatetime(N_Utils.getSecondTimestamp());
                course.setSchoolid(x.getSchoolid());

                for(Map.Entry<Integer, List<XzDateXjTeaVo>> entry : map.entrySet()){
                    List<XzDateXjTeaVo> xjTeas = entry.getValue();
                    String teachername = xjTeas.get(0).getTeachername();
                    //单个教师课程信息
                    String coursename = datazhou + dateKjVo.getKjname() +xjTeas.get(0).getSubjectname()+ "("+ teachername +")";

                    course.setCoursename(coursename);
                    course.setSubjectid(xjTeas.get(0).getSubjectid());
                    course.setTeacherid(entry.getKey());
                    course.setBegintime(dateKjVo.getBegintime());
                    course.setEndtime(dateKjVo.getEndtime());
                    course.setKjid(dateKjVo.getKjid());
                    //保存行政直播课程表
                    saveCourseAll(xjTeas,course,xzDateVo.getCurrentdate());
                }
            }
        }
        return new ResultVo(0, "");
    }

    //指定天某天某节课下的教师班级层级关系
    private Map<Integer, List<XzDateXjTeaVo>> listChangeTeacherClass(List<XzDateXjTeaVo> xjTealst) {
        Map<Integer, List<XzDateXjTeaVo>> map = new HashMap<>();

        for (int i = 0; i < xjTealst.size(); i++) {
            XzDateXjTeaVo xzDateXjTeaVo = xjTealst.get(i);
            if (map.containsKey(xzDateXjTeaVo.getTeacherid())) {
                map.get(xzDateXjTeaVo.getTeacherid()).add(xzDateXjTeaVo);
            } else {
                List<XzDateXjTeaVo> classlst = new ArrayList<>();
                classlst.add(xzDateXjTeaVo);
                map.put(xzDateXjTeaVo.getTeacherid(), classlst);
            }
        }
        return map;
    }

    //保存行政直播课程表
    @Transactional(rollbackFor = Exception.class)
    public void saveCourseAll(List<XzDateXjTeaVo> xjTeas,LCourse course,String currentdate){
        try {
            //获取集合中所有的班级id
            List classidlst = xjTeas.stream().map(XzDateXjTeaVo::getClassid).collect(Collectors.toList());
            List<Map> users = courseMapper.listUid(classidlst);//获取课班级下学生

            //新生成直播班班级信息
            TClass c = new TClass();
            c.setClassname(course.getCoursename() + "班");
            c.setCreateid(course.getCreateid());
            c.setSchoolid(course.getSchoolid());
            Integer year = schoolMapper.getYearById(course.getSchoolid());
            c.setSchoolyear(year);
            c.setYear(year);
            c.setClassorder(999);
            c.setGradeid(course.getGradeid());
            c.setClasstype(99);//已直播班级

            Integer classid = classService.save(c);//新生成直播班id
            course.setClassid(classid);

            //设置班级对应教师
            //班级教师
            TClassTeacher teacher = new TClassTeacher();
            teacher.setClassid(classid);
            teacher.setTeacherid(course.getTeacherid());
            teacher.setCreateid(course.getCreateid());
            teacher.setCreatetime(N_Utils.getSecondTimestamp());
            teacher.setSchoolid(course.getSchoolid());
            classTeacherMapper.insertSelective(teacher);

            //保存课程信息
            courseMapper.insertUseGeneratedKeys(course);
            Integer courseid = course.getId();//课程id

            List<Integer> studentids = new ArrayList<>();
            users.forEach(userVo -> {
                studentids.add((Integer) userVo.get("userid"));
            });

            classService.batchSaveStu(course.getCreateid(), course.getSchoolid(), classid, studentids);//批量添加报名学生
            courseMapper.updateStuNum(courseid);//更新已报学生数量

            //保存直播关联班级
            List<LCourseClass> cclist = new ArrayList<>();
            for(int i=0;i<classidlst.size();i++){
                Integer cid = Integer.parseInt(classidlst.get(i).toString());
                LCourseClass cc = new LCourseClass();
                cc.setCourseid(courseid);
                cc.setClassid(cid);
                cc.setCreateid(course.getCreateid());
                cc.setCreatetime(course.getCreatetime());
                cclist.add(cc);
                //通知班级
                ClassVo classVo = new ClassVo();
                classVo.setClassid(cid);
            }
            courseClassMapper.insertList(cclist);//保存原班级和直播的关联关系

            //获取班级人数
            ClassVo classVo = classService.getById(classid);

            //保存课程课节关联关系
            LCourseSection cs = new LCourseSection();
            cs.setCourseid(courseid);
            cs.setTeacherid(course.getTeacherid());
            cs.setSectionname(course.getCoursename());
            cs.setSectionorder(1);
            cs.setSectiondate(currentdate);
            cs.setPlanbegintime(course.getBegintime());
            cs.setPlanendtime(course.getEndtime());
            cs.setStunum(classVo.getClassnum());
            cs.setSectionstate(0);
            cs.setCreateid(course.getCreateid());
            cs.setCreatetime(course.getCreatetime());
            courseSectionMapper.insertUseGeneratedKeys(cs);//保存课程课节信息

        }catch (Exception e){
            log.info("保存行政直播课程表出错："+e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
    }

}
